import type { printJobs, printers } from "@/server/db/schema";
import { endOfMonth, startOfMonth } from "date-fns";
import type { InferSelectModel } from "drizzle-orm";

interface PrinterIdleTime {
	printerId: string;
	printerName: string;
	averageIdleTime: number; // in minutes
}

/**
 * Calculates the average idle time for each printer within the current month.
 *
 * @param printJobs - Array of print job objects.
 * @param printers - Array of printer objects.
 * @returns An array of PrinterIdleTime objects with average idle times.
 */
export function calculatePrinterIdleTime(
	pJobs: InferSelectModel<typeof printJobs>[],
	p: InferSelectModel<typeof printers>[],
): PrinterIdleTime[] {
	const now = new Date();
	const startOfThisMonth = startOfMonth(now);
	const endOfThisMonth = endOfMonth(now);
	const totalMinutesInMonth = 60 * 70 * 4; // 60min * 70h (35*2) * 4 Weeks

	const usedTimePerPrinter: Record<string, number> = pJobs.reduce(
		(acc, job) => {
			const jobStart = new Date(job.startAt);
			if (jobStart >= startOfThisMonth && jobStart <= endOfThisMonth) {
				acc[job.printerId] = (acc[job.printerId] || 0) + job.durationInMinutes;
			}
			return acc;
		},
		{} as Record<string, number>,
	);

	return p.map((printer) => {
		const usedTime = usedTimePerPrinter[printer.id] || 0;
		const idleTime = totalMinutesInMonth - usedTime;
		const averageIdleTime = idleTime < 0 ? 0 : idleTime; // Ensure no negative idle time

		return {
			printerId: printer.id,
			printerName: printer.name,
			averageIdleTime,
		};
	});
}
